function [W,v] = getWarpingAndValenceMatrix(p)
% Input
%  p:  [L x 2]     warping path of length L
%
% Output
%  W               sparse warping matrix of p
%  v               vector representing the diagonal of the valence matrix 
%
% References
% Warping and Valence matrices of a warping path have been introduced in [1].
%
% [1] Schultz, D., & Jain, B. (2018). Nonsmooth analysis and subgradient 
%     methods for averaging in dynamic time warping spaces. 
%     Pattern Recognition, 74, 340-358.
%
% Author: David Schultz, DAI-Lab, TU Berlin, Germany, 06-2020 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

L = length(p);

n = p(L,1);
m = p(L,2);
    
W = sparse(p(:,1),p(:,2),ones(L,1),n,m);

v = sum(W,2);
end