%
% Author: David Schultz, DAI-Lab, TU Berlin, Germany, 06-2020 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

init

% Define two sample time series
x = [0 0.1 -0.1 0 1 2 2.1 2.1 2 0 -3 -3.1 -2.9 0 1 0.9 0.7 0.6 0.6 0.7 0.8 0.8 0.5 0]';
x = interp1(1:length(x),x,linspace(1,length(x),20),'pchip');
 
y = [0 0.1 0.1 0 0.1 -0.3 0 0 1 2 2.1 2.1 1 0 -3 -3.1 -3 -0.1 0 1 0.8 0.9 0.6 0.7 0]';
y = interp1(1:length(y),y,linspace(1,length(y),20),'pchip');
 
% Store all sample time series in cell array (matrix and tensor forms are
% also supported, but time series of different length are only supported in 
% cell array form. For details see description of SSG or MM.)
X = {};
X{1} = x';
X{2} = y';

N = length(X);


% Euclidean mean (arithmetic mean)
mu_euclid = sum(cell2mat(X),2)/N;

% Compute DTW Means using two different algorithms, SSG and MM (DBA)
nIter = 15;
init_sesquence = 'euclidean_mean';
w = 3; % warping window size
[mu_ssg,f_ssg] = SSG(X,nIter,[],init_sesquence,w);
[mu_mm,f_mm] = MM(X,nIter,[],init_sesquence,w);


% Compare solution quality of SSG and MM in terms of Frechet variation
figure;
plot(0:length(f_ssg)-1,f_ssg,'r','linewidth',2); hold on;
plot(0:length(f_mm)-1,f_mm,'b','linewidth',2)
legend('SSG','MM (DBA)')
xlabel('Epoch')
ylabel('Frechet variation')
title('Optimization process of DTW mean algorithms (lower is better)')


% Plot sample and means
% (Interpretation: Both sample time series are very similar, but their 
% peaks and valley appear at differend positions in time. The Euclidean 
% mean does not account for such temporal variations and to some extent
% smoothes out the peaks and valleys. The DTW means represent the peaks and 
% valleys of the sample time series better, however, time information is 
% lost to some extent. The degree of time 'blurring' can be controlled by 
% the parameter w.)
figure;
plot(X{1},'color',[0.3,0.3,0.3]); hold on;
plot(mu_euclid,'g','linewidth',2)
plot(mu_ssg,'r','linewidth',2)
plot(mu_mm,'b','linewidth',2)
for i = 2:N
    plot(X{i},'color',[0.3,0.3,0.3]);
end
legend('Sample','Euclidean Mean','SSG','MM (DBA)')




