%
% Author: David Schultz, DAI-Lab, TU Berlin, Germany, 06-2020 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

init

% Generate dataset X with two classes, each having N examples

N = 10;
n = 50;

shape =  [0 5 10 5 0];

% class 1 contains the shape once
X1 = zeros(N,n);
for i = 1:N
    x = 0.5*randn(50,1);
    pos1 = max(1,randi(n-5));
    pos2 = max(1,randi(n-5));
    x(pos1:pos1+4) = shape;
    X1(i,:) = x;
end

% class 2 contains the shape twice
X2 = zeros(N,n);
for i = 1:N
    x = 0.5*randn(50,1);
    pos1 = max(1,randi(n-5));
    pos2 = max(1,randi(n-5));
    while abs(pos1-pos2) < 5
        pos2 = max(1,randi(n-5));
    end
    x(pos1:pos1+4) = shape;
    x(pos2:pos2+4) = shape;
    X2(i,:) = x;
end

% plot examples of the dataset
if N>=3
    figure;
    for i=1:2:6
        subplot(3,2,i);plot(X1(i,:));
        if i==1
            title('Class 1')
        end
        subplot(3,2,i+1);plot(X2(i,:))
        if i==1
            title('Class 2')
        end
    end
    sgtitle('Examples of dataset')
end

X = [X1;X2];
true_labels = [ones(1,N), 2*ones(1,N)];


k = 2;

%set up mean algorithm
nEpochs = 15;
eta = []; % use default
initSequence = 'random_sample';
w = []; % do not use waping window
mean_algo = @(X)SSG(X, nEpochs, eta, initSequence, w);

% run k-means wrt. dtw distance
[idx_dtw,Mu_dtw] = kmeans_dtw(X,k,[],mean_algo,w);


% run ordinary Euclidean k-means
[idx_euclid,Mu_euclid] = kmeans(X,k);



% compute accuracy of k-means labellings based on ground truth
acc_dtw = sum(true_labels==idx_dtw)/(2*N);
dtw_labelling_changed = 0;
if acc_dtw < 0.5 % change of labels equired
    idx_dtw = -idx_dtw+3;
    acc_dtw = 1-acc_dtw;
    dtw_labelling_changed = 1;
end

acc_euclid = sum(true_labels==idx_euclid')/(2*N);
euclid_labelling_changed = 0;
if acc_euclid < 0.5 % change of labels equired
    idx_euclid = -idx_euclid+3;
    acc_euclid = 1-acc_euclid;
    euclid_labelling_changed = 1;
end
    
display(['True labels                                        : ', num2str(true_labels)])
display(['k-means DTW labels (up to interchanging 1 and 2)   : ', num2str(idx_dtw)])
display(['k-means Euclid labels (up to interchanging 1 and 2): ', num2str(idx_euclid')])

fprintf('\nAccuracy k-means DTW = %.2f',acc_dtw)
fprintf('\nAccuracy k-means Euclid = %.2f\n',acc_euclid)



% plot means
figure;
for i = 1:k
    subplot(1,2,i);
    if euclid_labelling_changed
        plot(Mu_euclid(-i+3,:))
    else
        plot(Mu_euclid(i,:))
    end
    title(sprintf('Class %d',i))
end
sgtitle('k-means Euclid')

figure;
for i = 1:k
    subplot(1,2,i);
    if dtw_labelling_changed
        plot(Mu_dtw{-i+3})
    else
        plot(Mu_dtw{i})
    end
    title(sprintf('Class %d',i))
end
sgtitle('k-means DTW')


