%
% Author: David Schultz, DAI-Lab, TU Berlin, Germany, 10-2021 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

init

%% Define a dataset

% load ECG200 dataset (see data/ECG200/README.md for credits)
DATA_TEST = load('data/ECG200/ECG200_TEST.tsv');
DATA_TEST(:,1) = []; % Remove class labels.

DATA_TRAIN = load('data/ECG200/ECG200_TRAIN.tsv');
DATA_TRAIN(:,1) = []; % Remove class labels.

% we join TRAIN and TEST data
X_orig = [DATA_TRAIN; DATA_TEST];

[N,n] = size(X_orig);

% delete random sequences from the dataset
p = 0.1;    
max_seq_len = 10;
X = delete_sequences(X_orig, p, max_seq_len);
        
        
%% Configure and run the k-means algorithm with integrated data imputation

% Set up dtw mean algorithm (e.g. SSG or MM as provided here)
nEpochs = []; % use default
eta = []; % use default
initSequence = 'random_sample';
w = 10; % waping window
mean_algo = @(X)SSG(X, nEpochs, eta, initSequence, w);

% run k-means (wrt. dtw distance) with integrated data imputation
k = 4;
maxIter = 10;
[idx,Mu,X_imputed] = kmeans_dtw_imputation(X, k, [], maxIter, w, mean_algo);


%% Plot results

% plot centroids (the k means)
figure;
for i = 1:k
    subplot(1,k,i);
    plot(Mu{i});
    title(sprintf('Class %d',i));
end
sgtitle('Centroids')


% plot sample of imputed data
grid_size = 2;
P = randperm(N);
P = P(1:grid_size^2);

figure;
for i = 1:length(P)
   ind = P(i);
   subplot(grid_size,grid_size,i);
   hold on;
   plot(X_imputed{i},'r','linewidth',2)
   plot(X(i,:),'b','linewidth',2)
   plot(X_orig(i,:),'--k','linewidth',1)
   legend('estimated','original','ground truth')
end
sgtitle('Sample of imputed data')
